local self = StellarUF
local surf = self.surface


-- font for digit related text
local fontnum = "Interface\\AddOns\\StellarUF\\fonts\\font1.ttf"
-- font for player names
local fontname = "Interface\\AddOns\\StellarUF\\fonts\\font1.ttf"
-- font for player info
local fontinfo = "Interface\\AddOns\\StellarUF\\fonts\\font1.ttf"
local gtt = GameTooltip

-- basic layout parameters and locals
local layout = {
	full = { -- Player and Target frames
		frame = { w = 191, h = 50, },
		portrait = { s = 48, x = 1, y = -1, },
		hp = { w = 140, h = 33, x = 1, y = -1, fontsize = 10, },
		mp = { w = 126, h = 12, x = 0, y = -1, fontsize = 10, },
		classicon = { s = 12, x = -2, y = 0, },
		statusicon = { s = 15, x = -2, y = 0, },
		pvpicon = { s = 42, x = 5, y = -8, tarx = 9, },
		leadericon = { s = 12, x = 13, y = 0, },
		looticon = { s = 10, x = 24, y = 1, },
		auraicon = { s = 16, bx = 2, by = -51, bcount = 32, brows = 2, bcols = 16, dx = 2, dy = -68, dcount = 40, drows = 3, dcols = 16, },
	},
	medium = { -- Pet and Party frames
		frame = { w = 151, h = 34, },
		portrait = { s = 32, x = 1, y = -1, },
		hp = { w = 116, h = 22, x = 1, y = -1, fontsize = 7, },
		mp = { w = 107, h = 8, x = 0, y = -1, fontsize = 7, },
		classicon = { s = 8, x = -1, y = 0, },
		statusicon = { s = 12, x = -1, y = 0, },
		pvpicon = { s = 32, x = 4, y = -6, },
		leadericon = { s = 9, x = 8, y = 0, },
		looticon = { s = 8, x = 16, y = 1, },
		auraicon = { s = 12, bx = 32, by = -34, dx = 152, dy = -1, dcount = 8, drows = 2, dcols = 4, },
	},
	small = { -- TargetOfTarget and Focus frames
		frame = { w = 60, h = 18, },
		portrait = { s = 16, x = 1, y = -1, },
		hp = { w = 42, h = 12, x = 0, y = 0, fontsize = 7, },
		mp = { w = 42, h = 4, x = 0, y = 0, },
		auraicon = { s = 8, bx = 0, by = -18, bcount = 8, dcount = 8, dx = 0, dy = -26, },
	},
	tiny = { -- Party pet frames
		frame = { w = 35, h = 12, },
		portrait = { s = 12, x = 0, y = 0, },
		hp = { w = 23, h = 8, x = 0, y = -1, fontsize = 6, },
		mp = { w = 23, h = 2, x = 0, y = 0, },
	},
}



-- just some frame functions to reduce redundant lines of code

local function CreateBase(parent, n, w, h, backdrop, r, g, b, a, mouseable, texture)
	local f = CreateFrame((mouseable and "BUTTON") or "FRAME", nil, parent)
	f:ClearAllPoints()
	f:SetWidth(w)
	f:SetHeight(h)
	if backdrop then
		if texture then
			f:SetBackdrop({ bgFile = texture, })
		else
			f:SetBackdrop({ bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", tile = true, tileSize = 16, })
		end
		f:SetBackdropColor(r or 0, g or 0, b or 0, a or 1)
	end
	parent[n] = f
end

local function lCreateTexture(parent, n, w, h, overlay, inset, texture)
	local t = parent:CreateTexture(nil, overlay or "BACKGROUND")
	t:ClearAllPoints()
	
	t:SetWidth(w or 1)
	t:SetHeight(h or 1)
	if inset then
		t:SetPoint("TOPLEFT", parent, inset, -inset)
		t:SetPoint("BOTTOMRIGHT", parent, -inset, inset)
	end
	t:SetTexture(texture)
	parent[n] = t
end

local function CreateText(parent, n, w, h, font, fontsize, justifyH, justifyV, point, x, y, overlay, noshadow)
	local t = parent:CreateFontString(nil, overlay or "ARTWORK")
	t:SetJustifyH(justifyH)
	t:SetJustifyV(justifyV)
	t:SetFont(font, fontsize)
	t:SetHeight(h)
	t:SetWidth(w-2)
	t:ClearAllPoints()
	t:SetPoint(point, parent, point, x, y)
	if not noshadow then
		t:SetShadowColor(0.1, 0.1, 0.1)
		t:SetShadowOffset(1,-1)
	end
	t:SetTextColor(1, 1, 1)
	parent[n] = t
end

--****************************************************--
----------------
---- Icons -----
----------------

local function CreateIcon(parent, n, size, backdrop, mouseable)
	CreateBase(parent, n, size, size, backdrop, 0, 0, 0, 0.5, mouseable)
	lCreateTexture(parent[n], "texture", size, size, "OVERLAY", 0, nil)
end
local function AuraEnterB()
	gtt:SetOwner(this, "ANCHOR_BOTTOMRIGHT", 8, -16)
	gtt:SetUnitBuff(this:GetParent().unit, this.id)
	gtt:Show()
end
local function AuraEnterD()
	gtt:SetOwner(this, "ANCHOR_BOTTOMRIGHT", 8, -16)
	gtt:SetUnitDebuff(this:GetParent().unit, this.id)
	gtt:Show()
end
local function GTTHide() gtt:Hide() end

local function CreateAuraIcon(parent, n, size, type, id, notooltip)
	CreateBase(parent, n, size, size, true, 0, 0, 0, 1, true, "Interface\\AddOns\\StellarUF\\images\\backdrop")
	local f = parent[n]
	lCreateTexture(f, "icon", size-2, size-2, "ARTWORK", 1, nil)
	f.icon:ClearAllPoints()
	f.icon:SetPoint("BOTTOMLEFT")
	f.icon:SetPoint("TOPRIGHT", f, -1.2, -1.2)
	f.icon:SetTexCoord(0.07, 0.93, 0.07, 0.93)
	CreateText(f, "count", size+2, size*(5/8), fontnum, size*(5/8), "RIGHT", "BOTTOM", "BOTTOMRIGHT", 0, 0, "OVERLAY")
	
	f.id = id
	if not notooltip then
		f:SetScript("OnEnter", (type == "buff" and AuraEnterB) or AuraEnterD)
		f:SetScript("OnLeave", GTTHide)
	end
end

local function CreateAuraGroup(parent, n, size, count, type, rows, cols)
	for i=1,count,1 do
		local index = format("%s%d", n, i)
		local tindex = format("%s%d", n, i-1)
		CreateAuraIcon(parent, index, size, type, i)
		parent[index]:Hide()
		if i ~= 1 then
			local prevrow
			if rows and cols and count > cols then
				prevrow = (floor(i / cols) * cols) - cols + 1
			end
			if prevrow and math.fmod(i - 1, cols) == 0 then
				parent[index]:SetPoint("TOP", parent[n..prevrow], "BOTTOM", 0, 0)
			else
				parent[index]:SetPoint("LEFT", parent[tindex], "RIGHT", 0, 0)
			end
		end
	end
end


--****************************************************--
-------------------
---- Portraits ----
-------------------

-- portrait glow effects
local function PortraitGlow()
	local g = this.glow
	if g.duration > 0 then
		g.duration = g.duration - arg1
		g.elapsed = g.elapsed + arg1
		if g.elapsed > 0.6 then
			g.sign = -g.sign
			g.elapsed = 0
		end
		g.alpha = min(1, g.alpha + (g.sign * arg1))
		g.alpha = max(0.4, g.alpha)
		g:SetAlpha(g.alpha)
	else
		g:SetAlpha(0)
	end
end
local function PortraitShow()
	this:SetCamera(0) 
	this:SetModelScale(2)
end
local function CreatePortrait(parent, size, type)
	CreateBase(parent, "portrait", size, size)
	local p = parent.portrait
	if type == "full" or type == "medium" then
		p:SetBackdrop({ bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", tile = true, tileSize = 16,
		                edgeFile = "", edgeSize = 0, })
		p:SetBackdropColor(0.1, 0.1, 0.1, 0.8)
		p.d3 = CreateFrame("PlayerModel", nil, p)
		p.d3:ClearAllPoints()
		p.d3:SetPoint("TOPLEFT", 0.5, -0.5)
		p.d3:SetPoint("BOTTOMRIGHT", 0.5, -0.5)
		p.d3:SetScript("OnShow", PortraitShow)
		CreateText(p, "combattext", size*1.5, size/4, fontnum, size/4, "CENTER", "TOP", "TOP", 0, 2, "OVERLAY")
		p.d3:SetFrameLevel(2)
		p.combattext.alpha = 0
		
		lCreateTexture(p, "glow", size, size, "ARTWORK", 0, "Interface\\AddOns\\StellarUF\\images\\portraitglow")
		p.glow.sign = 1
		p.glow.alpha = 0
		p.glow.duration = 0
		p.glow.elapsed = 0
		p.glow:SetAlpha(0)
		p:SetScript("OnUpdate", PortraitGlow)
	end
	lCreateTexture(p, "d2", nil, nil, "BORDER", 0)
end


--****************************************************--
--------------------
---- Statusbars ----
--------------------
local function BackBarFader()
	local b = this.backbar
	if b.alpha > 0 then
		b.alpha = max(b.alpha - arg1, 0)
		b:SetAlpha(min(b.alpha, 1))
	end
end
-- custom statusbar creation
local function CreateStatusBar(parent, n, w, h, reverse, type)
	local left, right = (not reverse and "LEFT") or "RIGHT", (not reverse and "RIGHT") or "LEFT"
	local texture = StellarUF.surface:Fetch(StellarUF.db.profile.texture)
	CreateBase(parent, n, w, h, true, 0, 0, 0, 1, nil, texture)
	local f = parent[n]
	-- main bar
	lCreateTexture(f, "bar", w, h, "ARTWORK", nil, texture)
	f.bar:SetPoint(left, f, left, 0, 0)
	if type == "full" or type == "medium" then
		-- bar loss fader
		lCreateTexture(f, "backbar", w, h, "BORDER", nil, texture)
		f.backbar:SetPoint(left, f, left, 0, 0)
		f.backbar.alpha = 0.001
		f.backbar:SetAlpha(0)
		f:SetScript("OnUpdate", BackBarFader)
	end
end

local function CreateHealthBar(parent, w, h, fontsize, reverse, type)
	local left, right, rev = (not reverse and "LEFT") or "RIGHT", (not reverse and "RIGHT") or "LEFT", (not reverse and 1) or -1
	CreateStatusBar(parent, "hp", w, h, reverse, type)
	local f, c = parent.hp, self.hpcolor
	f:SetAlpha(0.95)
	f.bar:SetVertexColor(c.r, c.g, c.b, 1)
	f:SetBackdropColor(0, c.g/3.5, 0, 0.6)
	if type == "full" or type == "medium" then
		f.backbar:SetVertexColor(1, 0.2, 0.2)
		CreateText( f, "nametext", w, fontsize+4.2, fontname, fontsize+3.8, 
		                 left, "TOP", "TOP"..left, 1 * rev, 1 )
		CreateText( f, "infotext", w, fontsize+1, fontinfo, fontsize+2,
		                 left, "TOP", "TOP"..left, 2 * rev, -(fontsize+1) )
		CreateText( f, "hptext", w, h, fontnum, fontsize, 
		                 right, "BOTTOM", "BOTTOM"..right, -1.2 * rev, 1.2, nil, true )
		CreateText( f, "statustext", w, fontsize+4, fontinfo, fontsize+4,
		                 "CENTER", "CENTER", "BOTTOM", 0, -4 , "OVERLAY")
		f.statustext:SetVertexColor(0.5, 0.5, 0.5, 0.7)
	elseif type == "small" then
		CreateText( f, "nametext", w, fontsize+2, fontname, fontsize+1, 
		                 left, "TOP", "TOP"..left, 0, 1, nil, true )
		CreateText( f, "hptext", w, h, fontnum, fontsize, 
		                 "CENTER", "BOTTOM", "CENTER", 0, -2, nil, true )
	elseif type == "tiny" then
		CreateText( f, "hptext", w, h, fontnum, fontsize, 
		                 "CENTER", "BOTTOM", "CENTER", 0, -1, nil, true)
	end
end

local function CreateManaBar(parent, w, h, fontsize, reverse, type)
	local left, right, rev = (not reverse and "LEFT") or "RIGHT", (not reverse and "RIGHT") or "LEFT", (not reverse and 1) or -1
	CreateStatusBar(parent, "mp", w, h, reverse, type)
	parent.mp:SetAlpha(0.95)
	parent.mp:SetBackdropColor(0.2, 0.2, 0.2, 0.6)
	if type == "full" or type == "medium" then
		parent.mp.backbar:SetVertexColor(0.8, 0.8, 0.8, 0.95)
		CreateText( parent.mp, "mptext", w, h, fontnum, fontsize, 
		                 right, "BOTTOM", "BOTTOM"..right, -1.2 * rev, 1.2, nil, true )
	end
end


--****************************************************--

local function ClassIconOnEnter()
-- class icon tooltip (shows sex, race, and faction)
	gtt:SetOwner(this, "ANCHOR_BOTTOMRIGHT", 4, -4)
	local u = this:GetParent().unit
	local sex = UnitSex(u) -- 1 for N/A, 2 for Male, 3 for Female
	local _, race = UnitRace(u)
	local faction = UnitFactionGroup(u)
	
	if sex == 2 then 
		sex = "Male"
	elseif sex == 3 then 
		sex = "Female"
	else 
		sex = "Unknown" 
	end
	
	if faction then 
		faction = format("(%s)", faction) 
	end
	if not race and not UnitIsPlayer(u) then 
		race = "NPC" 
	end
	 
	gtt:SetText(format("%s %s %s", sex, race or "", faction or ""), 1, 1, 1) 
	gtt:Show()
end

-- cheap flash effect for dispellable debuffs
local function UpDebuffAlert()
	this.elapsed = this.elapsed + arg1
	if this.elapsed > 0.6 then
		this.sign = -this.sign
		this.elapsed = 0
	end
	this.alpha = min(0.55, this.alpha + (this.sign * arg1))
	this.alpha = max(this.alpha, 0.15)
	this:SetAlpha(this.alpha)
end

-- use blizz's default unit dropdown menus
local dropdown = {
	player = PlayerFrameDropDown,
	target = TargetFrameDropDown,
	party1 = PartyMemberFrame1DropDown,
	party2 = PartyMemberFrame2DropDown,
	party3 = PartyMemberFrame3DropDown,
	party4 = PartyMemberFrame4DropDown,
	pet = PetFrameDropDown,
}

-- puts this mess together
local function MainEnter()
	UnitFrame_OnEnter() 
	this:SetBackdropColor(1, 1, 0.5, 0.8)
end
local function MainLeave()
	UnitFrame_OnLeave() 
	this:SetBackdropColor(0, 0, 0, 0.2)
end
local function MainShow()
	self:UpUnit(this.unit, true)
end
function StellarUF:CreateUnitFrame(unit, size, reverse)
	local left, right = (not reverse and "LEFT") or "RIGHT", (not reverse and "RIGHT") or "LEFT"
	local rev = (not reverse and 1) or -1
	local lay = layout[size]
	local db = self.db.profile
	
	-- baseframe
	self.units[unit] = CreateFrame("BUTTON", "StellarUF"..unit, UIParent, "SecureUnitButtonTemplate")
	local f = self.units[unit]
	f.unit = unit
	f.frametype = size
	f.reversed = reverse
	f:SetWidth(lay.frame.w)
	f:SetHeight(lay.frame.h)
	
	f:SetBackdrop({ bgFile = surf:Fetch(db.texture), })
	f:SetBackdropColor(0, 0, 0, 0.2)
	
	f:RegisterForClicks("LeftButtonUp", "RightButtonUp")
	f:ClearAllPoints()
	f:SetPoint("TOPLEFT", UIParent, "TOPLEFT", db.pos[unit].x, db.pos[unit].y)
	f:SetScale(db.scale[size] or 1.0)
	f:SetScript("OnEnter", MainEnter)
	f:SetScript("OnLeave", MainLeave)
	f:SetClampedToScreen(true)
	
	-- portrait
	CreatePortrait(f, lay.portrait.s, size)
	f.portrait:SetPoint("TOP"..left, f, "TOP"..left, lay.portrait.x * rev, lay.portrait.y)
	
	-- healthbar
	CreateHealthBar(f, lay.hp.w, lay.hp.h, lay.hp.fontsize, reverse, size)
	f.hp:SetPoint("TOP"..left, f.portrait, "TOP"..right, lay.hp.x * rev, lay.hp.y)
	
	-- powerbar
	CreateManaBar(f, lay.mp.w, lay.mp.h, lay.mp.fontsize, reverse, size)
	f.mp:SetPoint("TOP"..right, f.hp, "BOTTOM"..right, lay.mp.x * rev, lay.mp.y)
	
	-- create a bunch of info icons
	if size == "full" or size == "medium" then
		CreateIcon(f, "classicon", lay.classicon.s, true)
		f.classicon:EnableMouse(true)
		f.classicon:SetPoint("TOP"..right, f.mp, "TOP"..left, lay.classicon.x * rev, lay.classicon.y)
		f.classicon:SetScript("OnEnter", ClassIconOnEnter)
		f.classicon:SetScript("OnLeave", GTTHide)
		
		if unit ~= "target" then
			CreateIcon(f, "statusicon", lay.statusicon.s)
			f.statusicon:SetPoint("CENTER", f.portrait, "BOTTOM"..left, lay.statusicon.x * rev, lay.statusicon.y)
		end
		
		CreateIcon(f, "pvpicon", lay.pvpicon.s)
		f.pvpicon:SetPoint("CENTER", f.portrait, left, (unit == "target" and lay.pvpicon.tarx) or lay.pvpicon.x, lay.pvpicon.y)
		
		CreateIcon(f, "leadericon", lay.leadericon.s)
		f.leadericon.texture:SetTexture("Interface\\GroupFrame\\UI-Group-LeaderIcon")
		f.leadericon:SetPoint("CENTER", f.portrait, "BOTTOM"..left, lay.leadericon.x * rev, lay.leadericon.y)
		
		CreateIcon(f, "looticon", lay.looticon.s)
		f.looticon.texture:SetTexture("Interface\\GroupFrame\\UI-Group-MasterLooter")
		f.looticon:SetPoint("CENTER", f.portrait, "BOTTOM"..left, lay.looticon.x * rev, lay.looticon.y)

		-- create debuff alert
		if self.cls ~= "WARRIOR" and self.cls ~= "ROGUE" and self.cls ~= "HUNTER" and self.cls ~= "WARLOCK" then
			CreateAuraIcon(f, "debuffalert", lay.frame.h * 0.8, nil, nil, true)
			local da = f.debuffalert
			da:SetFrameStrata("HIGH")
			da:SetPoint(right, f, right, rev * 2, 0)
			da.icon:ClearAllPoints()
			da.icon:SetPoint("TOPRIGHT", da, -4, -4)
			da.icon:SetPoint("BOTTOMLEFT", da, 2, 2)
			da.elapsed = 0
			da.alpha = 0
			da.sign = 1
			da:SetScript("OnUpdate", UpDebuffAlert)
			da:SetAlpha(0)
			da:Hide()
		end
	end
	
	-- create buff/debuff icons (not for player or tiny mode frames)
	if unit ~= "player" and lay.auraicon then
		local la = lay.auraicon
		CreateAuraGroup(f, "buff", la.s, la.bcount or 16, "buff", la.brows, la.bcols)
		f.buff1:SetPoint("TOPLEFT", f, "TOPLEFT", la.bx, la.by)
		
		-- make party debuffs bigger than normal
		local debuffsize = la.s
		if strfind(unit, "party") then 
			debuffsize = la.s + 4 
		end
		CreateAuraGroup(f, "debuff", debuffsize, la.dcount or 16, "debuff", la.drows, la.dcols)
		f.debuff1:SetPoint("TOPLEFT", f, "TOPLEFT", la.dx, la.dy)
	end
	
	-- create group number text on player/target frames
	if unit == "player" or unit == "target" then
		CreateBase(f, "group", 13, 13, true, 0.3, 0.3, 0.3, 0.8)
		CreateText( f.group, "text", 13, 13, fontnum, 10, 
		                 "CENTER", "CENTER", "CENTER", 0, 0 )
		f.group:SetPoint("TOP", f.leadericon, "CENTER", 0, 0)
		f.group:Hide()
	end
	
	-- pet happiness icon for hunters (just recycled blizz's)
	if unit == "pet" and self.cls == "HUNTER" then
		local ph = PetFrameHappiness
		ph:SetParent(f)
		ph:ClearAllPoints()
		ph:SetPoint("TOP", f.portrait, "BOTTOM", 5, 0)
		ph:Show()
		ph:SetWidth(16)
		ph:SetHeight(15.5)
		ph:RegisterEvent("UNIT_HAPPINESS")
		ph:SetScript("OnEvent", PetFrame_SetHappiness)
	end
	
	-- target only stuff
	if unit == "target" then self:TargetOnlyStuff(f) end
	
	-- puts the word "Focus" above the focus frame
	if unit == "focus" then
		CreateText( f, "title", 30, 10, fontnum, 8, 
		                 "CENTER", "BOTTOM", "BOTTOM", 0, 18, "ARTWORK" )
		f.title:SetText("|cffffff00Focus|r")
	end

	-- make focustarget a child of focus
	if unit == "focustarget" then
		f:ClearAllPoints()
		f:SetParent(self.units.focus or UIParent)
		f:SetPoint("LEFT", self.units.focus, "RIGHT", 1, 0)
	end

	f:SetAlpha(0.9)
	
	--Set Secure attributes
	if not dropdown[unit] then
		SecureUnitButton_OnLoad(f, unit)
	else
		SecureUnitButton_OnLoad(f, unit, function() ToggleDropDownMenu(1, nil, dropdown[unit], f, lay.frame.w / 2, 0) end)
	end
	if unit ~= "player" then
		f:Hide()
		RegisterUnitWatch(f, false)
	end
	
	-- this is click casting support? ok
	ClickCastFrames = ClickCastFrames or {}
	ClickCastFrames[f] = true
	
	f:SetScript("OnShow", MainShow)
end

-- just a chunk of code taken out of the above
local function UpCombo()
	local points = GetComboPoints()
	if points > 0 then
		local offsetr = points * 0.2
		local offsetl = offsetr - 0.2
		this.texture:SetTexCoord(offsetl, offsetr, 0, 0.5)
		this.glow:SetTexCoord(offsetl, offsetr, 0.5, 1)
		this:Show()
	else
		this:Hide()
	end
end
local function UpComboGlow()
	this.elapsed = this.elapsed + arg1
	if this.elapsed > 0.7 then
		this.sign = -this.sign
		this.elapsed = 0
	end
	this.alpha = min(0.95, this.alpha + (this.sign * arg1))
	this.alpha = max(this.alpha, 0.45)
	this.glow:SetAlpha(this.alpha)
end
local function InspectClick()
	if not UnitIsVisible("target") then return end
	if arg1 == "LeftButton" then 
		InspectUnit("target") 
	elseif arg1 == "RightButton" then 
		if not DressUpFrame:IsShown() then 
			ShowUIPanel(DressUpFrame) 
		end
		DressUpModel:SetUnit("target")
	end 
end
local function InspectEnter()
	gtt:SetOwner(this, "ANCHOR_BOTTOMRIGHT")
	gtt:SetText("Inspect")
	gtt:AddLine(" <Left-click> to inspect.\n <Right-click> to dressup.")
	gtt:Show()
end

function StellarUF:TargetOnlyStuff(f)
	
	--combo points
	if self.cls == "ROGUE" or self.cls == "DRUID" then
		CreateBase(f, "combo", 29, 18)
		local c = f.combo
		c:SetFrameStrata("HIGH")
		lCreateTexture(c, "texture", 29, 18, "OVERLAY", 0, "Interface\\AddOns\\StellarUF\\images\\combo")
		lCreateTexture(c, "glow", 29, 18, "ARTWORK", 0, "Interface\\AddOns\\StellarUF\\images\\combo")
		c.alpha = 0.5
		c.elapsed = 0
		c.sign = 1
		c:SetPoint("CENTER", f.portrait, "BOTTOMLEFT", 18, 4)
		c:RegisterEvent("PLAYER_TARGET_CHANGED")
		c:RegisterEvent("PLAYER_COMBO_POINTS")
		c:SetScript("OnEvent", UpCombo)
		c:SetScript("OnUpdate", UpComboGlow)
		c:Hide()
	end
	
	--raid target icon
	CreateIcon(f, "raidtarget", 16)
	f.raidtarget:SetPoint("CENTER", f.portrait, "TOPLEFT", 2, 2)
	f.raidtarget.texture:SetTexture("Interface\\TargetingFrame\\UI-RaidTargetingIcons")
	f.raidtarget.texture:SetAlpha(0.8)

	-- inspect button
	f.inspect = CreateFrame("BUTTON", nil, f)
	f.inspect:SetWidth(16)
	f.inspect:SetHeight(16)
	f.inspect:SetPoint("CENTER", f, "BOTTOMRIGHT", -1, 1)
	f.inspect:SetFrameStrata("HIGH")
	f.inspect:SetPushedTexture("Interface\\AddOns\\StellarUF\\images\\inspectdown")
	f.inspect:SetNormalTexture("Interface\\AddOns\\StellarUF\\images\\inspectup")
	f.inspect:SetHighlightTexture("Interface\\Minimap\\UI-Minimap-ZoomButton-Highlight")
	f.inspect:SetScript("OnMouseUp", InspectClick)
	f.inspect:SetScript("OnEnter", InspectEnter)
	f.inspect:SetScript("OnLeave", GTTHide)
end

-- position
function StellarUF:SavePosition(unit)
	local a,b,c,d,e = self.units[unit]:GetPoint()
	if a == "TOPLEFT" and c == "TOPLEFT" then
		self.db.profile.pos[unit].x = floor(d + 0.5)
		self.db.profile.pos[unit].y = floor(e + 0.5)
	end
end

-- make movable or not
function StellarUF:ToggleDrag()
	if self.unlocked then
		for u,f in pairs(self.units) do
			if f.frametype ~= "tiny" and u ~= "focustarget" then
				f:SetMovable(false)
				f:RegisterForDrag()
			end
		end
		self.unlocked = false
	else
		for u,f in pairs(self.units) do
			if f.frametype ~= "tiny" and u ~= "focustarget" then
				f:SetMovable(true)
				f:RegisterForDrag("LeftButton")
				f:SetScript("OnDragStart", function() if not InCombatLockdown() then this:StartMoving() end end)
				f:SetScript("OnDragStop", function() this:StopMovingOrSizing() self:SavePosition(this.unit) end)
			end
		end
		self.unlocked = true
	end
end

